//******************************************************************************
// OpenGL Radiosity example unit by Georgy Moshkin
// -----------------------------------------------
// email  : tmtlib@narod.ru
// WWW    : http://www.tmtlib.narod.ru/
//
//
// Description:
// ------------
// Radiosity lightmaps generation with OpenGL. Non physical correct.
// Calculation speed depends on FPS of your OpenGL video card.
//
// License:
// --------
// This unit is freely distributable without licensing fees and is
// provided without guarantee or warrantee expressed or implied. This unit
// is Public Domain. Feel free to use or enhance this code. You can use the
// program for any purpose you see fit.
//
//******************************************************************************
unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls,
  OpenGL,
  glBMP,   // TEXTURES UNIT
  _dream,  // RADIOSITY UNIT
  _vector; // VECTOR MATH UNIT

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    Timer1: TTimer;
    RadioGroup1: TRadioGroup;
    GroupBox1: TGroupBox;
    ScrollBar1: TScrollBar;
    Button1: TButton;
    ListBox1: TListBox;
    Button2: TButton;
    Button3: TButton;

    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TimerDraw(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
  private
    { Private declarations }
    rc : HGLRC;    // RC
    dc  : HDC;     // DC

    procedure InitGL;
    procedure glDraw;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////


procedure TForm1.glDraw();
var i,j:integer;
begin

 glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  glLoadIdentity();

  glPolygonmode(GL_FRONT_AND_BACK, GL_FILL);

  gluLookAt(0,0,-100, //  
            0,0,0,    //  
            0,1,0);    //    

  glColor3f(1,1,1);

  glRotatef(GetTickCount/100,0,1,0);

  if leveldim>0 then renderquad;

  SwapBuffers(DC);

end;

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
procedure TForm1.InitGL;
begin
  glClearColor(0.0, 0.0, 0.0, 1.0); 	   // White Background
  glShadeModel(GL_SMOOTH);                 // Enables Smooth Color Shading
  glClearDepth(1.0);                       // Depth Buffer Setup
  glEnable(GL_DEPTH_TEST);                 // Enable Depth Buffer
  glDepthFunc(GL_LESS);		           // The Type Of Depth Test To Do

  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);   //Realy Nice perspective calculations

  ScrollBar1.SetParams(3,1,8);
  leveldim:=0;

  ListBox1.Items.Add('Don''t touch anything after clicking');
  ListBox1.Items.Add('"3.Calculate Radiosity" button.');
  ListBox1.Items.Add('Don''t move window during calculation!');
  ListBox1.Items.Add('WAIT SOME TIME!!!');
  ListBox1.Items.Add(' ');
  ListBox1.Items.Add('For "4-dots" example set iterations');
  ListBox1.Items.Add('number to maximum level.');

end;

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

procedure TForm1.FormCreate(Sender: TObject);
var pfd : TPIXELFORMATDESCRIPTOR;
    pf  : Integer;
begin
  DecimalSeparator:='.';
  dc:=GetDC(Panel3.Handle);

  pfd.nSize:=sizeof(pfd);
  pfd.nVersion:=1;
  pfd.dwFlags:=PFD_DRAW_TO_WINDOW or PFD_SUPPORT_OPENGL or PFD_DOUBLEBUFFER or 0;
  pfd.iPixelType:=PFD_TYPE_RGBA;
  pfd.cColorBits:=32;

  pf :=ChoosePixelFormat(dc, @pfd);
  SetPixelFormat(dc, pf, @pfd);

  rc :=wglCreateContext(dc);    // RC
  wglMakeCurrent(dc,rc);        // DC

  InitGL;
end;

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

procedure TForm1.FormResize(Sender: TObject);
begin
  glViewport(0, 0, Panel3.Width, Panel3.Height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0, Panel3.Width/Panel3.Height, 0.1, 500.0);

  glMatrixMode(GL_MODELVIEW);
  glDraw();
end;

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

procedure TForm1.FormDestroy(Sender: TObject);
begin
  wglMakeCurrent(0,0);
  wglDeleteContext(rc);
end;

procedure TForm1.TimerDraw(Sender: TObject);
begin
glDraw();
end;

procedure MakeRoom;
begin

//    ():

    with level[0] do begin
     p1.x:=-25; p1.y:=-25; p1.z:=-25;
     p2.x:= 25; p2.y:=-25; p2.z:=-25;
     p3.x:= 25; p3.y:= 25; p3.z:=-25;
     p4.x:=-25; p4.y:= 25; p4.z:=-25;
     normal:=NormalVector(p1,p2,p3);
    end;


    with level[1] do begin
     p1.x:=-25; p1.y:=-25; p1.z:=25;
     p2.x:=-25; p2.y:= 25; p2.z:=25;
     p3.x:= 25; p3.y:= 25; p3.z:=25;
     p4.x:= 25; p4.y:=-25; p4.z:=25;
     normal:=NormalVector(p1,p2,p3);
    end;


    with level[2] do begin
     p1.x:=-25; p1.y:=-25; p1.z:=-25;
     p2.x:=-25; p2.y:=-25; p2.z:= 25;
     p3.x:= 25; p3.y:=-25; p3.z:= 25;
     p4.x:= 25; p4.y:=-25; p4.z:=-25;
     normal:=NormalVector(p1,p2,p3);
    end;

  with level[3] do begin
     p1.x:=-25; p1.y:=25; p1.z:=-25;
     p2.x:= 25; p2.y:=25; p2.z:=-25;
     p3.x:= 25; p3.y:=25; p3.z:= 25;
     p4.x:=-25; p4.y:=25; p4.z:= 25;
     normal:=NormalVector(p1,p2,p3);
    end;

    with level[4] do begin
     p1.z:=-25; p1.y:=-25; p1.x:=25;
     p2.z:= 25; p2.y:=-25; p2.x:=25;
     p3.z:= 25; p3.y:= 25; p3.x:=25;
     p4.z:=-25; p4.y:= 25; p4.x:=25;
     normal:=NormalVector(p1,p2,p3);
    end;


    with level[5] do begin
     p1.z:=-25; p1.y:=-25; p1.x:=-25;
     p2.z:=-25; p2.y:= 25; p2.x:=-25;
     p3.z:= 25; p3.y:= 25; p3.x:=-25;
     p4.z:= 25; p4.y:=-25; p4.x:=-25;
     normal:=NormalVector(p1,p2,p3);
    end;

//    ():

    with level[6] do begin
     p1.x:=-15; p1.y:=-25; p1.z:=0;
     p2.x:= 15; p2.y:=-25; p2.z:=0;
     p3.x:= 15; p3.y:= 25; p3.z:=0;
     p4.x:=-15; p4.y:= 25; p4.z:=0;
     normal:=NormalVector(p1,p2,p3);
    end;

    with level[7] do begin
     p1.x:=-15; p1.y:=-25; p1.z:=0;
     p2.x:=-15; p2.y:= 25; p2.z:=0;
     p3.x:= 15; p3.y:= 25; p3.z:=0;
     p4.x:= 15; p4.y:=-25; p4.z:=0;
     normal:=NormalVector(p1,p2,p3);
    end;


    //     :

    leveldim:=7; // 7    8 : 0,1,2,3,4,5,6,7

end;

procedure TForm1.Button1Click(Sender: TObject);
const ts=16; //    
var i,j,k:integer; //  
var mytex:string; //  ;
begin

//  :

 case RadioGroup1.ItemIndex of
  0: mytex:='lt.bmp'; //  1
  1: mytex:='blue.bmp'; //  2
  2: mytex:='dots.bmp'; //  3
 end;

//   :

   level[0].map.Create(mytex);
   level[0].map.SetTextureWrap(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE);
   level[0].map.GenTexture(False, False);

   level[1].map.InitEmpty(ts,ts);
   level[1].map.SetTextureWrap(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE);
   level[1].map.GenTexture(False, False);

   level[2].map.InitEmpty(ts,ts);
   level[2].map.SetTextureWrap(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE);
   level[2].map.GenTexture(False, False);

   level[3].map.InitEmpty(ts,ts);
   level[3].map.SetTextureWrap(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE);
   level[3].map.GenTexture(False, False);

   level[4].map.InitEmpty(ts,ts);
   level[4].map.SetTextureWrap(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE);
   level[4].map.GenTexture(False, False);

   level[5].map.InitEmpty(ts,ts);
   level[5].map.SetTextureWrap(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE);
   level[5].map.GenTexture(False, False);

//   :

   level[6].map.InitEmpty(ts,ts);
   level[6].map.SetTextureWrap(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE);
   level[6].map.GenTexture(False, False);

   level[7].map.InitEmpty(ts,ts);
   level[7].map.SetTextureWrap(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE);
   level[7].map.GenTexture(False, False);

//    ר  (   ):

  for k:=1 to 7 do
   for i:=0 to level[1].map.getWidth-1 do
    for j:=0 to level[1].map.getHeight-1 do
     level[k].map.SetPixel(i,j,0,0,0);

    makeroom;

    //    CULLING:

    glEnable(GL_TEXTURE_2D);
    glEnable(GL_CULL_FACE);

    // ٨   OPENGL :

   level[0].map.GenTexture(False, False);
   level[1].map.GenTexture(False, False);
   level[2].map.GenTexture(False, False);
   level[3].map.GenTexture(False, False);
   level[4].map.GenTexture(False, False);
   level[5].map.GenTexture(False, False);
   level[6].map.GenTexture(False, False);
   level[7].map.GenTexture(False, False);

   // -= RADIOSITY BY GEORGY MOSHKIN =-

   radiositymode(32,32); //   Radosity
   CreateRadiosityMem;   //    "" Radiosity
   calculateFormFactor;  //   FormFactor  

   //  RADIOSITY:

   for i:=1 to ScrollBar1.Position do
    begin
     RadiosityStep; //    RADIOSITY
    end;


   //   RADIOSITY:

   RadiosityEnd(Panel3.Width, Panel3.Height);

end;

procedure TForm1.Button2Click(Sender: TObject);
var i:integer;
begin

listbox1.Clear;
// SAVE LIGHTMAPS to light1.bmp, light2.bmp, light3.bmp, ...
for i:=0 to leveldim do
 begin
   level[i].map.SaveImage('light'+inttostr(i)+'.bmp');
   listBox1.Items.Add('light'+inttostr(i)+'.bmp saved!')
 end;


end;


procedure TForm1.Button3Click(Sender: TObject);
var i:integer;
begin

listbox1.Clear;

// MAKE 3d ROOM
makeroom;

    glEnable(GL_TEXTURE_2D);
    glEnable(GL_CULL_FACE);


// LOAD LIGHTMAPS from bmp files
for i:=0 to leveldim do
 begin
   level[i].map.LoadImage('light'+inttostr(i)+'.bmp');
   level[i].map.SetTextureWrap(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE);
   level[i].map.GenTexture(False, False);
 end;
   listBox1.Items.Add('Loaded ok!');
end;

// This code is public domain, feel free to do with it any descent thing.
end.

