//******************************************************************************
// OpenGL Radiosity unit by Georgy Moshkin
// ---------------------------------------
// email  : tmtlib@narod.ru
// WWW    : http://www.tmtlib.narod.ru/
//
// Description:
// ------------
// Radiosity lightmaps generation with OpenGL. Non physical correct.
// Calculation speed depends on FPS of your OpenGL video card.
//
// License:
// --------
// This unit is freely distributable without licensing fees and is
// provided without guarantee or warrantee expressed or implied. This unit
// is Public Domain. Feel free to use or enhance this code. You can use the
// program for any purpose you see fit.
//
//******************************************************************************
unit _dream; // _DREAM.PAS RADIOSITY UNIT

interface

uses opengl,  //  OpenGL
     _vector, //   
     windows, // WinAPI
     glbmp;   //   

//        
// Ψ  tmtlib.narod.ru

type TRGB = record         //      RGB
             r,g,b:single; // (r - , g - , b - )
            end;

// ---------------------------------------------
// HELP     _dream.pas:
//      e-mail tmtlib@narod.ru
// Georgy Moshkin (http://www.tmtlib.narod.ru)

// RADIOSITYMODE PROCEDURE - INITIALISATION
procedure RadiosityMode(xw,xh:integer);
//   Radiosity ( viewport  XW  XH)
//   xw=xh. : 8x8, 16x16, 32x32, 64x64  ..
// /Example: RadiosityMode(32,32);
// :  XW  XH      ViewPort-,
//       ,  ""  .

// RADIOSITYEND PROCEDURE - VIEWPORT RESTORE
procedure RadiosityEnd(width,height:integer);
//     ViewPort  
// width, height. ,     OpenGL : 640x480
// /Example: RadiosityEnd(640,480);

// CREATERADIOSITYMEM PROCEDURE - CREATE STORAGE FOR VIEWPORT PIXELS
procedure CreateRadiosityMem;
//  ,     ,  "" 
//  ViewPort-  Radiosity.     XW  XH,
//    RadiosityMode(32,32);
// (  GetMem(RadiosityPointer, RadW*RadH*3); )
// /Example: CreateRadiosityTexture;
// :      
// Radiosity . RADIOSITY LIGHTMAPS GENERATION SPEED
// DEPENDS ON xw,xh

// CALCULATEFORMFACTOR PROCEDURE - CALCULATES FORM-FACTOR
procedure CalculateFormFactor;
//   Form-Factor,    
//   Radiosity.   Form-Factor  
//  ViewPort-  RadiosityTexture.
// : CalculateFormFactor;
// :     Form-Factor-, 
//    -.

// RADIOSITYSTEP PROCEDURE - DO ONE RADIOSITY-STEP
procedure RadiosityStep;
//       Radiosity,
//        
// : for i:=1 to ... do RadiosityStep;
// :     .

// RENDERQUAD PROCEDURE - RENDER SCENE
procedure RenderQuad;
//       level
//      Radiosity,
//       
// : RenderQuad;

var RadW, RadH:integer; // ,    
                        // Radiosity- ( RadiosityPointer, ViewPort-
                        //  ).


type TQuad=record               //   
           p1,p2,p3,p4:TVector; //  
           normal:TVector;      // 
           map:TGLBMP;          //  
           tex:GLUINT;          // OpenGL-  
           end;

var Textures:array[0..10] of TGLBMP; // 11   TGLBMP
var level:array[0..10] of TQUAD;     // 11 

leveldim:integer; //     
                  //        0  11

var RadiosityPointer: Pointer; //      Radiosity

var formFactor:array[0..512-1, 0..512-1] of single; // -
                                                    //  
                                                    // RadW x RadH 



implementation

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


procedure RenderQuad; //  

var i:integer; //  

begin

 for i:=0 to leveldim do //    
 with level[i] do
  begin
   map.bind;          //   

   glBegin(GL_QUADS);

   glTexCoord2f(0,0);           //  
   glVertex3f(p1.x,p1.y,p1.z);  //  

   glTexCoord2f(1,0);
   glVertex3f(p2.x,p2.y,p2.z);

   glTexCoord2f(1,1);
   glVertex3f(p3.x,p3.y,p3.z);

   glTexCoord2f(0,1);
   glVertex3f(p4.x,p4.y,p4.z);

   glEnd;
  end;
end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure RadiosityMode(xw,xh:integer);  //  Radiosity
begin
  RadW:=xw; //   
  RadH:=xh;

  Randomize; //     

  glViewport(0, 0, RadW, RadH); //   viewport

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(90,RadW/RadH,0.1,1000);  //   90  !!!
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure RadiosityEnd(width,height:integer); //    Radiosity
begin
 glViewport(0, 0, width, height); //  ViewPort
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0, width/height, 0.1, 1000.0); //   ,  = 45 !!!
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
end;

//     .   - 
function adj(inp:single):single;
var z:single;
begin
z:=inp;                   //     (R,G  B)
if (z>0) and (z<0.5) then // ,    
z:=sin(z*3.14/0.5);       //    
end;


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure updateRadiosityTexture; //  ViewPort
var zzz:integer;
begin
  //    Z-BUFFER-
  glReadPixels(0, 0, RadW, RadH, GL_RGB, GL_UNSIGNED_BYTE, RadiosityPointer);
  // Ѩ! VIEWPORT   RadiosityPointer
end;


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

//   ,  
//     Form-Factor-
// (   RadiosityPointer)
function RadiosityColor:Trgb;

type TPixel = Record   // 
       R, G, B : Byte; //     SINGLE,   BYTE
     end;              // R,G,B - , ,   (0...255)

var I, J : Integer;    //  
    Pixel : ^TPixel;   // ,      
                       //   Tpixel- 

    R,G,B:single;      //    SINGLE (0...1)

begin

R:=0; // 
G:=0;
B:=0;

  for I :=0 to RadW-1 do    //     RadiosityPointer
    for J :=0 to RadH-1 do
    begin
      Pixel:=Pointer(Integer(RadiosityPointer) + 3*(i*RadW + j));
      R:=R+Pixel^.R * formFactor[i,j]; //  FormFactor
      G:=G+Pixel^.G * formFactor[i,j]; //    R,G,B
      B:=B+Pixel^.B * formFactor[i,j]; //   *
    end;

    result.R:=R/(RadW*RadH*255); //   Single
    result.G:=G/(RadW*RadH*255); //  0...255  0...1
    result.B:=B/(RadW*RadH*255); //     RadW*RadH

end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure CreateRadiosityMem; //    "" Radiosity
begin
  GetMem(RadiosityPointer, RadW*RadH*3); // 3 -    
end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure calculateFormFactor; //   -
var i,j:integer;      //  
begin

//  FormFactor-     i,j
//    :
for i:=0 to RadW-1 do
 for j:=0 to RadH-1 do
    formFactor[i,j]:=0.5+0.5*sin( i*3.14/(RadW-1) )*sin( j*3.14/(RadH-1) );

end;


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//      "" Radiosity
//   Viewport.
function xrender(v_eye,v_dir,v_up:Tvector):TRGB; //   
begin
//    :
// v_eye -   (     )
// v_dir -   (  )
// v_up -   (,    )
//  :
//  ,    

    glClearColor(0.0,0.0,0.0,1); //    (0,0,0)
    glEnable(GL_DEPTH_TEST);     //  DEPTH TEST (Z-BUFFER)
    glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);  //    Z-BUFFER
    glLoadIdentity(); //  

    //      ( ):
                  gluLookAt(v_eye.x+v_dir.x/1000,v_eye.y+v_dir.y/1000,v_eye.z+v_dir.z/1000,
                            v_eye.x+v_dir.x,
                            v_eye.y+v_dir.y,
                            v_eye.z+v_dir.z,
                            v_up.x,v_up.y,v_up.z);

    //  :
    renderquad;

    //   VIEWPORT-    RADIOSITYPOINTER:
    updateRadiosityTexture;

    //      RADIOSITYPOINTER  ר FORMFACTOR:
    xrender:=RadiosityColor;

end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

//  RADIOSITY (  )
procedure RadiosityStep;

var randvec:Tvector; //  
    perpvec:Tvector; //  

    v_eye,v_dir,v_up:Tvector; //  ,  
                              //   

    i,j,k,l:integer; //  

    basevector:TVector; //   ( )

    //     :
    uvector:TVector; //  U-
    vvector:TVector; //  V-

    u,v:integer; //      

    ulen,vlen:single; //    

    udx,vdx:single; //      uvector  vvector

    temprgb:TRGB; //     

begin


for i:=0 to leveldim do //    
 with level[i] do
 begin
  basevector:=p1;               //   =   

  uvector:=vecminusvec(p2,p1);  //   
  vvector:=vecminusvec(p4,p1);

  ulen:=VectorLen(uvector); //   
  vlen:=VectorLen(vvector);

  udx:=ulen/(map.getWidth{-1}); //     
  vdx:=vlen/(map.getHeight{-1});

  uvector:=Normalize(uvector); //    ()
  vvector:=Normalize(vvector);

  //      :

  BaseVector:=VecPlusVec(basevector,

            VecPlusVec(Umnoj(uvector,udx/2),
                       Umnoj(vvector,vdx/2))
                       );

  // Ĩ       :

  for u:=0 to map.getWidth-1 do
   for v:=0 to map.getHeight-1 do
    begin

     randvec.x:=127-random(255); //  
     randvec.y:=127-random(255);
     randvec.z:=127-random(255);
     randvec:=Normalize(randvec); // 

     perpvec:=(CrossVector(normal,randvec)); //    ( )

     //   ,    
     v_eye:=VecPlusVec(basevector,

            VecPlusVec(Umnoj(uvector,udx*u),
                       Umnoj(vvector,vdx*v))
                       );

     //   =  
     v_dir:=normal;

     //  
     v_up:=perpvec;

     //        TEMPRGB
     TempRGB:=xrender(v_eye,v_dir,v_up);

     //    ,    
     map.AddPixel(u,v,round(255*TempRGB.R),
                      round(255*TempRGB.G),
                      round(255*TempRGB.B));


    end;



 end;


//  OPENGL- 
for i:=0 to leveldim do
 with level[i] do
  map.GenTexture(False, False);
end;


// This code is public domain, feel free to do with it any descent thing.
// Author: Georgy V. Moshkin
end.
